@@ -1,10 +1,10 @@
-/* $Cambridge: exim/exim-src/exim_monitor/em_globals.c,v 1.7 2005/05/23 15:28:38 fanf2 Exp $ */
+/* $Cambridge: exim/exim-src/exim_monitor/em_globals.c,v 1.9 2006/02/07 11:18:59 ph10 Exp $ */
 
 /*************************************************
 *                Exim Monitor                    *
 *************************************************/
 
-/* Copyright (c) University of Cambridge 1995 - 2005 */
+/* Copyright (c) University of Cambridge 1995 - 2006 */
 /* See the file NOTICE for conditions of use and distribution. */
 
 
@@ -118,7 +118,7 @@ that fires up the monitor fishes the value out by using -bP anyway. */
 #endif
 
 
-uschar *acl_var[ACL_C_MAX+ACL_M_MAX];
+uschar *acl_var[ACL_CVARS + ACL_MVARS];
 
 uschar *active_hostname        = NULL;
 BOOL    allow_unqualified_recipient = FALSE;
