@@ -1,8 +1,10 @@
+/* $Cambridge: exim/exim-src/exim_monitor/em_queue.c,v 1.4 2006/02/14 14:26:15 ph10 Exp $ */
+
 /*************************************************
 *                 Exim Monitor                   *
 *************************************************/
 
-/* Copyright (c) University of Cambridge 1995 - 2002 */
+/* Copyright (c) University of Cambridge 1995 - 2006 */
 /* See the file NOTICE for conditions of use and distribution. */
 
 
@@ -17,10 +19,10 @@ queue and displaying the data therefrom. */
 addresses. Otherwise, define a macro that does nothing. */
 
 #ifdef ANONYMIZE
-static char *anon(char *s)
+static uschar *anon(uschar *s)
 {
-static char anon_result[256];
-char *ss = anon_result;
+static uschar anon_result[256];
+uschar *ss = anon_result;
 for (; *s != 0; s++) *ss++ = (*s == '@' || *s == '.')? *s : 'x';
 *ss = 0;
 return anon_result;
@@ -38,7 +40,7 @@ static int queue_total = 0;   /* number of items in queue */
 
 /* Table for turning base-62 numbers into binary */
 
-static char tab62[] =
+static uschar tab62[] =
           {0,1,2,3,4,5,6,7,8,9,0,0,0,0,0,0,     /* 0-9 */
            0,10,11,12,13,14,15,16,17,18,19,20,  /* A-K */
           21,22,23,24,25,26,27,28,29,30,31,32,  /* L-W */
@@ -63,22 +65,30 @@ if it is dest_remove, remove if present and return NULL. The
 address is lowercased to start with, unless it begins with
 "*", which it does for error messages. */
 
-dest_item *find_dest(queue_item *q, char *name, int action)
+dest_item *find_dest(queue_item *q, uschar *name, int action, BOOL caseless)
 {
 dest_item *dd;
 dest_item **d = &(q->destinations);
-char *s = name;
-
-if (*s != '*') while (*s) { *s = tolower(*s); s++; }
 
 while (*d != NULL)
   {
-  if (strcmp(name, (*d)->address) == 0)
+  if ((caseless? strcmpic(name,(*d)->address) : Ustrcmp(name,(*d)->address))
+        == 0)
     {
+    dest_item *ddd;
+
     if (action != dest_remove) return *d;
     dd = *d;
     *d = dd->next;
     store_free(dd);
+
+    /* Unset any parent pointers that were to this address */
+
+    for (ddd = q->destinations; ddd != NULL; ddd = ddd->next)
+      {
+      if (ddd->parent == dd) ddd->parent = NULL;
+      }
+
     return NULL;
     }
   d = &((*d)->next);
@@ -86,8 +96,8 @@ while (*d != NULL)
 
 if (action != dest_add) return NULL;
 
-dd = (dest_item *)store_malloc(sizeof(dest_item) + (int)strlen(name));
-strcpy(dd->address, name);
+dd = (dest_item *)store_malloc(sizeof(dest_item) + Ustrlen(name));
+Ustrcpy(dd->address, name);
 dd->next = NULL;
 dd->parent = NULL;
 *d = dd;
@@ -118,20 +128,20 @@ store_free(p);
 *             Set up new queue item              *
 *************************************************/
 
-static queue_item *set_up(char *name, int dir_char)
+static queue_item *set_up(uschar *name, int dir_char)
 {
 int i, rc, save_errno;
 struct stat statdata;
 void *reset_point;
-char *p;
+uschar *p;
 queue_item *q = (queue_item *)store_malloc(sizeof(queue_item));
-char buffer[256];
+uschar buffer[256];
 
 /* Initialize the block */
 
 q->next = q->prev = NULL;
 q->destinations = NULL;
-strcpy(q->name, name);
+Ustrcpy(q->name, name);
 q->seen = TRUE;
 q->frozen = FALSE;
 q->dir_char = dir_char;
@@ -151,7 +161,7 @@ spool_read_header that is to be preserved is copied into malloc store. */
 reset_point = store_get(0);
 message_size = 0;
 message_subdir[0] = dir_char;
-sprintf(buffer, "%s-H", name);
+sprintf(CS buffer, "%s-H", name);
 rc =  spool_read_header(buffer, FALSE, TRUE);
 save_errno = errno;
 
@@ -181,13 +191,13 @@ otherwise it sets set up in the sender field. */
 
 if (rc != spool_read_OK)
   {
-  char *msg;
+  uschar *msg;
 
   if (save_errno == ERRNO_SPOOLFORMAT)
     {
     struct stat statbuf;
-    sprintf(big_buffer, "%s/input/%s", spool_directory, buffer);
-    if (stat(big_buffer, &statbuf) == 0)
+    sprintf(CS big_buffer, "%s/input/%s", spool_directory, buffer);
+    if (Ustat(big_buffer, &statbuf) == 0)
       msg = string_sprintf("*** Format error in spool file: size = %d ***",
         statbuf.st_size);
     else msg = string_sprintf("*** Format error in spool file ***");
@@ -196,12 +206,13 @@ if (rc != spool_read_OK)
 
   if (rc == spool_read_hdrerror)
     {
-    (void)find_dest(q, msg, dest_add);
+    (void)find_dest(q, msg, dest_add, FALSE);
     }
   else
     {
     deliver_freeze = FALSE;
     sender_address = msg;
+    recipients_count = 0;
     }
   }
 
@@ -213,51 +224,41 @@ if (sender_set_untrusted)
   {
   if (sender_address[0] == 0)
     {
-    q->sender = store_malloc((int)strlen(originator_login) + 6);
-    sprintf(q->sender, "<> (%s)", originator_login);
+    q->sender = store_malloc(Ustrlen(originator_login) + 6);
+    sprintf(CS q->sender, "<> (%s)", originator_login);
     }
   else
     {
-    q->sender = store_malloc((int)strlen(sender_address) +
-      (int)strlen(originator_login) + 4);
-    sprintf(q->sender, "%s (%s)", sender_address, originator_login);
+    q->sender = store_malloc(Ustrlen(sender_address) +
+      Ustrlen(originator_login) + 4);
+    sprintf(CS q->sender, "%s (%s)", sender_address, originator_login);
     }
   }
 else
   {
-  q->sender = store_malloc((int)strlen(sender_address) + 1);
-  strcpy(q->sender, sender_address);
+  q->sender = store_malloc(Ustrlen(sender_address) + 1);
+  Ustrcpy(q->sender, sender_address);
   }
 
 sender_address = NULL;
 
-sprintf(buffer, "%s/input/%s/%s-D", spool_directory, message_subdir, name);
-if (stat(buffer, &statdata) == 0) q->size = message_size + statdata.st_size;
+sprintf(CS buffer, "%s/input/%s/%s-D", spool_directory, message_subdir, name);
+if (Ustat(buffer, &statdata) == 0)
+  q->size = message_size + statdata.st_size - SPOOL_DATA_START_OFFSET + 1;
 
 /* Scan and process the recipients list, skipping any that have already
-been delivered, and removing visible names. In the nonrecipients list,
-domains are lower-cased. */
+been delivered, and removing visible names. */
 
 if (recipients_list != NULL)
   {
   for (i = 0; i < recipients_count; i++)
     {
-    BOOL delivered = TRUE;
-    char *r = recipients_list[i].address;
+    uschar *r = recipients_list[i].address;
     if (tree_search(tree_nonrecipients, r) == NULL)
-      {
-      char *rr = r;
-      while (*rr != 0 && *rr != '@') rr++;
-      while (*rr != 0) { *rr = tolower(*rr); rr++; }
-      if (tree_search(tree_nonrecipients, r) == NULL)
-        delivered = FALSE;
-      }
-
-    if (!delivered)
       {
       if ((p = strstric(r+1, qualify_domain, FALSE)) != NULL &&
         *(--p) == '@') *p = 0;
-      (void)find_dest(q, r, dest_add);
+      (void)find_dest(q, r, dest_add, FALSE);
       }
     }
   }
@@ -310,7 +311,7 @@ while (p != NULL)
 
 
 
-queue_item *find_queue(char *name, int action, int dir_char)
+queue_item *find_queue(uschar *name, int action, int dir_char)
 {
 int first = 0;
 int last = queue_index_size - 1;
@@ -335,7 +336,7 @@ if (queue_total == 0)
 /* Also handle insertion at the start or end of the queue
 as special cases. */
 
-if (strcmp(name, (queue_index[0])->name) < 0)
+if (Ustrcmp(name, (queue_index[0])->name) < 0)
   {
   if (action != queue_add) return NULL;
   if ((qq = set_up(name, dir_char)) != NULL)
@@ -349,7 +350,7 @@ if (strcmp(name, (queue_index[0])->name) < 0)
   return NULL;
   }
 
-if (strcmp(name, (queue_index[queue_index_size-1])->name) > 0)
+if (Ustrcmp(name, (queue_index[queue_index_size-1])->name) > 0)
   {
   if (action != queue_add) return NULL;
   if ((qq = set_up(name, dir_char)) != NULL)
@@ -368,7 +369,7 @@ when the name is somewhere in the middle, if present. */
 
 while (middle > first)
   {
-  if (strcmp(name, (queue_index[middle])->name) >= 0) first = middle;
+  if (Ustrcmp(name, (queue_index[middle])->name) >= 0) first = middle;
     else last = middle;
   middle = (first + last)/2;
   }
@@ -382,7 +383,7 @@ q = queue_index[last];
 
 for (;;)
   {
-  int c = strcmp(name, p->name);
+  int c = Ustrcmp(name, p->name);
 
   /* Already on queue; mark seen if required. */
 
@@ -448,14 +449,14 @@ int indexptr = 1;
 queue_item *p;
 struct dirent *ent;
 DIR *dd;
-char input_dir[256];
-char subdirs[64];
+uschar input_dir[256];
+uschar subdirs[64];
 
 subdirs[0] = 0;
 stripchart_total[0] = 0;
 
-sprintf(input_dir, "%s/input", spool_directory);
-subptr = (int)strlen(input_dir);
+sprintf(CS input_dir, "%s/input", spool_directory);
+subptr = Ustrlen(input_dir);
 input_dir[subptr+2] = 0;               /* terminator for lengthened name */
 
 /* Loop for each spool file on the queue - searching any subdirectories that
@@ -471,19 +472,19 @@ for (i = 0; i < subdir_max; i++)
     input_dir[subptr+1] = subdirchar;
     }
 
-  dd = opendir(input_dir);
+  dd = opendir(CS input_dir);
   if (dd == NULL) continue;
 
   while ((ent = readdir(dd)) != NULL)
     {
-    char *name = ent->d_name;
-    int len = (int)strlen(name);
+    uschar *name = US ent->d_name;
+    int len = Ustrlen(name);
 
     /* If we find a single alphameric sub-directory on the first
     pass, add it to the list for subsequent scans, and remember that
     we are dealing with a split directory. */
 
-    if (i == 0 && len == 1 && isalnum((uschar)*name))
+    if (i == 0 && len == 1 && isalnum(*name))
       {
       subdirs[subdir_max++] = *name;
       spool_is_split = TRUE;
@@ -496,10 +497,10 @@ for (i = 0; i < subdir_max; i++)
         name[SPOOL_NAME_LENGTH - 2] == '-' &&
         name[SPOOL_NAME_LENGTH - 1] == 'H')
       {
-      char basename[SPOOL_NAME_LENGTH];
+      uschar basename[SPOOL_NAME_LENGTH + 1];
       stripchart_total[0]++;
       if (!eximon_initialized) { printf("."); fflush(stdout); }
-      strcpy(basename, name);
+      Ustrcpy(basename, name);
       basename[SPOOL_NAME_LENGTH - 2] = 0;
       if (full) find_queue(basename, queue_add, subdirchar);
       }
@@ -579,7 +580,7 @@ if (tn != NULL)
   {
   if (tn->left != NULL) scan_tree(p, tn->left);
   if (tn->right != NULL) scan_tree(p, tn->right);
-  (void)find_dest(p, tn->name, dest_remove);
+  (void)find_dest(p, tn->name, dest_remove, FALSE);
   }
 }
 
@@ -591,16 +592,16 @@ int i;
 FILE *jread;
 void *reset_point;
 struct stat statdata;
-char buffer[1024];
+uschar buffer[1024];
 
 message_subdir[0] = p->dir_char;
 
-sprintf(buffer, "%s/input/%s/%s-J", spool_directory, message_subdir, p->name);
-jread = fopen(buffer, "r");
+sprintf(CS buffer, "%s/input/%s/%s-J", spool_directory, message_subdir, p->name);
+jread = fopen(CS buffer, "r");
 if (jread == NULL)
   {
-  sprintf(buffer, "%s/input/%s/%s-H", spool_directory, message_subdir, p->name);
-  if (stat(buffer, &statdata) < 0 || p->update_time == statdata.st_mtime)
+  sprintf(CS buffer, "%s/input/%s/%s-H", spool_directory, message_subdir, p->name);
+  if (Ustat(buffer, &statdata) < 0 || p->update_time == statdata.st_mtime)
     return;
   }
 
@@ -608,7 +609,7 @@ if (jread == NULL)
 Arrange to recover the dynamic store afterwards. */
 
 reset_point = store_get(0);
-sprintf(buffer, "%s-H", p->name);
+sprintf(CS buffer, "%s-H", p->name);
 if (spool_read_header(buffer, FALSE, TRUE) != spool_read_OK)
   {
   store_reset(reset_point);
@@ -620,11 +621,11 @@ if (spool_read_header(buffer, FALSE, TRUE) != spool_read_OK)
 
 if (jread != NULL)
   {
-  while (fgets(big_buffer, big_buffer_size, jread) != NULL)
+  while (Ufgets(big_buffer, big_buffer_size, jread) != NULL)
     {
-    int n = (int)strlen(big_buffer);
+    int n = Ustrlen(big_buffer);
     big_buffer[n-1] = 0;
-    tree_add_nonrecipient(big_buffer+1, big_buffer[0] == 'Y');
+    tree_add_nonrecipient(big_buffer);
     }
   fclose(jread);
   }
@@ -637,15 +638,15 @@ if (recipients_list != NULL)
   {
   for (i = 0; i < recipients_count; i++)
     {
-    char *pp;
-    char *r = recipients_list[i].address;
+    uschar *pp;
+    uschar *r = recipients_list[i].address;
     tree_node *node = tree_search(tree_nonrecipients, r);
 
     if (node == NULL)
       {
-      char temp[256];
-      char *rr = temp;
-      strcpy(temp, r);
+      uschar temp[256];
+      uschar *rr = temp;
+      Ustrcpy(temp, r);
       while (*rr != 0 && *rr != '@') rr++;
       while (*rr != 0) { *rr = tolower(*rr); rr++; }
       node = tree_search(tree_nonrecipients, temp);
@@ -654,9 +655,9 @@ if (recipients_list != NULL)
     if ((pp = strstric(r+1, qualify_domain, FALSE)) != NULL &&
       *(--pp) == '@') *pp = 0;
     if (node == NULL)
-      (void)find_dest(p, r, dest_add);
+      (void)find_dest(p, r, dest_add, FALSE);
     else
-      (void)find_dest(p, r, dest_remove);
+      (void)find_dest(p, r, dest_remove, FALSE);
     }
   }
 
@@ -696,7 +697,7 @@ while (p != NULL)
   {
   int count = 1;
   dest_item *dd, *ddd;
-  char u = 'm';
+  uschar u = 'm';
   int t = (now - p->input_time)/60;  /* minutes on queue */
 
   if (t > 90)
@@ -737,7 +738,7 @@ while (p != NULL)
     int len_address;
 
     if (ddd->address[0] == '*') break;
-    len_address = (int)strlen(ddd->address);
+    len_address = Ustrlen(ddd->address);
 
     for (skp = &queue_skip; ; skp = &(sk->next))
       {
@@ -759,9 +760,9 @@ while (p != NULL)
 
       /* If this address matches the skip item, break (sk != NULL) */
 
-      len_skip = (int)strlen(sk->text);
+      len_skip = Ustrlen(sk->text);
       if (len_skip <= len_address &&
-          strcmp(ddd->address + len_address - len_skip, sk->text) == 0)
+          Ustrcmp(ddd->address + len_address - len_skip, sk->text) == 0)
         break;
       }
 
@@ -778,18 +779,18 @@ while (p != NULL)
       t, u,
       string_format_size(p->size, big_buffer),
       p->name,
-      (p->sender == NULL)? "       " :
-        (p->sender[0] == 0)? "<>     " : anon(p->sender));
+      (p->sender == NULL)? US"       " :
+        (p->sender[0] == 0)? US"<>     " : anon(p->sender));
 
     text_showf(queue_widget, "%s%s%s",
       (dd == NULL || dd->address[0] == '*')? "" : "<",
-      (dd == NULL)? "" : anon(dd->address),
+      (dd == NULL)? US"" : anon(dd->address),
       (dd == NULL || dd->address[0] == '*')? "" : ">");
 
     if (dd != NULL && dd->parent != NULL && dd->parent->address[0] != '*')
       text_showf(queue_widget, " parent <%s>", anon(dd->parent->address));
 
-    text_show(queue_widget, "\n");
+    text_show(queue_widget, US"\n");
 
     if (dd != NULL) dd = dd->next;
     while (dd != NULL && count++ < queue_max_addresses)
@@ -798,7 +799,7 @@ while (p != NULL)
         anon(dd->address));
       if (dd->parent != NULL && dd->parent->address[0] != '*')
         text_showf(queue_widget, " parent <%s>", anon(dd->parent->address));
-      text_show(queue_widget, "\n");
+      text_show(queue_widget, US"\n");
       dd = dd->next;
       }
     if (dd != NULL)
