@@ -1,8 +1,10 @@
+/* $Cambridge: exim/exim-src/src/auths/plaintext.c,v 1.5 2006/02/23 12:41:22 ph10 Exp $ */
+
 /*************************************************
 *     Exim - an Internet mail transport agent    *
 *************************************************/
 
-/* Copyright (c) University of Cambridge 1995 - 2002 */
+/* Copyright (c) University of Cambridge 1995 - 2006 */
 /* See the file NOTICE for conditions of use and distribution. */
 
 #include "../exim.h"
@@ -12,6 +14,8 @@
 /* Options specific to the plaintext authentication mechanism. */
 
 optionlist auth_plaintext_options[] = {
+  { "client_ignore_invalid_base64", opt_bool,
+      (void *)(offsetof(auth_plaintext_options_block, client_ignore_invalid_base64)) },
   { "client_send",        opt_stringptr,
       (void *)(offsetof(auth_plaintext_options_block, client_send)) },
   { "server_condition",   opt_stringptr,
@@ -26,12 +30,13 @@ address can appear in the tables drtables.c. */
 int auth_plaintext_options_count =
   sizeof(auth_plaintext_options)/sizeof(optionlist);
 
-/* Default private options block for the contidion authentication method. */
+/* Default private options block for the plaintext authentication method. */
 
 auth_plaintext_options_block auth_plaintext_option_defaults = {
   NULL,              /* server_condition */
   NULL,              /* server_prompts */
-  NULL               /* client_send */
+  NULL,              /* client_send */
+  FALSE              /* client_ignore_invalid_base64 */
 };
 
 
@@ -62,27 +67,40 @@ if (ob->client_send != NULL) ablock->client = TRUE;
 /* For interface, see auths/README */
 
 int
-auth_plaintext_server(auth_instance *ablock, char *data)
+auth_plaintext_server(auth_instance *ablock, uschar *data)
 {
 auth_plaintext_options_block *ob =
   (auth_plaintext_options_block *)(ablock->options_block);
-char *prompts = ob->server_prompts;
-char *clear, *cond, *end, *s;
+uschar *prompts = ob->server_prompts;
+uschar *clear, *cond, *end, *s;
 int number = 1;
 int len, rc;
 int sep = 0;
 
+/* Expand a non-empty list of prompt strings */
+
+if (prompts != NULL)
+  {
+  prompts = expand_string(prompts);
+  if (prompts == NULL)
+    {
+    auth_defer_msg = expand_string_message;
+    return DEFER;
+    }
+  }
 
 /* If data was supplied on the AUTH command, decode it, and split it up into
-multiple items at binary zeros. If the data consists of the string "=" it
-indicates a single, empty string. */
+multiple items at binary zeros. The strings are put into $auth1, $auth2, etc,
+up to a maximum. To retain backwards compatibility, they are also put int $1,
+$2, etc. If the data consists of the string "=" it indicates a single, empty
+string. */
 
 if (*data != 0)
   {
-  if (strcmp(data, "=") == 0)
+  if (Ustrcmp(data, "=") == 0)
     {
-    expand_nstring[++expand_nmax] = "";
-    expand_nstring[expand_nmax] = 0;
+    auth_vars[0] = expand_nstring[++expand_nmax] = US"";
+    expand_nlength[expand_nmax] = 0;
     }
   else
     {
@@ -90,6 +108,7 @@ if (*data != 0)
     end = clear + len;
     while (clear < end && expand_nmax < EXPAND_MAXN)
       {
+      if (expand_nmax < AUTH_VARS) auth_vars[expand_nmax] = clear;
       expand_nstring[++expand_nmax] = clear;
       while (*clear != 0) clear++;
       expand_nlength[expand_nmax] = clear++ - expand_nstring[expand_nmax];
@@ -106,61 +125,74 @@ while ((s = string_nextinlist(&prompts, &sep, big_buffer, big_buffer_size))
         != NULL && expand_nmax < EXPAND_MAXN)
   {
   if (number++ <= expand_nmax) continue;
-  if ((rc = auth_get_data(&data, s)) != OK) return rc;
+  if ((rc = auth_get_data(&data, s, Ustrlen(s))) != OK) return rc;
   if ((len = auth_b64decode(data, &clear)) < 0) return BAD64;
   end = clear + len;
-  while (clear < end && expand_nmax < EXPAND_MAXN)
+
+  /* This loop must run at least once, in case the length is zero */
+  do
     {
+    if (expand_nmax < AUTH_VARS) auth_vars[expand_nmax] = clear;
     expand_nstring[++expand_nmax] = clear;
     while (*clear != 0) clear++;
     expand_nlength[expand_nmax] = clear++ - expand_nstring[expand_nmax];
     }
+  while (clear < end && expand_nmax < EXPAND_MAXN);
   }
 
-/* We now have a number of items of data in $1, $2, etc. Match against the
-decoded data by expanding the condition. Also expand the id to set if
-authentication succeeds. */
+/* We now have a number of items of data in $auth1, $auth2, etc (and also, for
+compatibility, in $1, $2, etc). Match against the decoded data by expanding the
+condition. */
 
 cond = expand_string(ob->server_condition);
 
-HDEBUG(2)
+HDEBUG(D_auth)
   {
   int i;
   debug_printf("%s authenticator:\n", ablock->name);
+  for (i = 0; i < AUTH_VARS; i++)
+    {
+    if (auth_vars[i] != NULL)
+      debug_printf("  $auth%d = %s\n", i + 1, auth_vars[i]);
+    }
   for (i = 1; i <= expand_nmax; i++)
     debug_printf("  $%d = %.*s\n", i, expand_nlength[i], expand_nstring[i]);
+  debug_print_string(ablock->server_debug_string);    /* customized debug */
   if (cond == NULL)
     debug_printf("expansion failed: %s\n", expand_string_message);
   else
     debug_printf("expanded string: %s\n", cond);
   }
 
-/* Expansion failure always yields DEFER, which will cause a temporary error
-code to be returned to the AUTH command. The problem is at the server end, so
-the client should try again later. */
+/* A forced expansion failure causes authentication to fail. Other expansion
+failures yield DEFER, which will cause a temporary error code to be returned to
+the AUTH command. The problem is at the server end, so the client should try
+again later. */
 
 if (cond == NULL)
   {
+  if (expand_string_forcedfail) return FAIL;
   auth_defer_msg = expand_string_message;
   return DEFER;
   }
 
 /* Return FAIL for empty string, "0", "no", and "false"; return OK for
 "1", "yes", and "true"; return DEFER for anything else, with the string
-available as an error text. */
+available as an error text for the user. */
 
 if (*cond == 0 ||
-    strcmp(cond, "0") == 0 ||
-    strcmpic(cond, "no") == 0 ||
-    strcmpic(cond, "false") == 0)
+    Ustrcmp(cond, "0") == 0 ||
+    strcmpic(cond, US"no") == 0 ||
+    strcmpic(cond, US"false") == 0)
   return FAIL;
 
-if (strcmp(cond, "1") == 0 ||
-    strcmpic(cond, "yes") == 0 ||
-    strcmpic(cond, "true") == 0)
+if (Ustrcmp(cond, "1") == 0 ||
+    strcmpic(cond, US"yes") == 0 ||
+    strcmpic(cond, US"true") == 0)
   return OK;
 
 auth_defer_msg = cond;
+auth_defer_user_msg = string_sprintf(": %s", cond);
 return DEFER;
 }
 
@@ -178,15 +210,16 @@ auth_plaintext_client(
   smtp_inblock *inblock,                 /* connection inblock */
   smtp_outblock *outblock,               /* connection outblock */
   int timeout,                           /* command timeout */
-  char *buffer,                          /* buffer for reading response */
+  uschar *buffer,                        /* buffer for reading response */
   int buffsize)                          /* size of buffer */
 {
 auth_plaintext_options_block *ob =
   (auth_plaintext_options_block *)(ablock->options_block);
-char *text = ob->client_send;
-char *s;
+uschar *text = ob->client_send;
+uschar *s;
 BOOL first = TRUE;
 int sep = 0;
+int auth_var_idx = 0;
 
 /* The text is broken up into a number of different data items, which are
 sent one by one. The first one is sent with the AUTH command; the remainder are
@@ -194,8 +227,9 @@ sent in response to subsequent prompts. Each is expanded before being sent. */
 
 while ((s = string_nextinlist(&text, &sep, big_buffer, big_buffer_size)) != NULL)
   {
-  int i, len;
-  char *ss = expand_string(s);
+  int i, len, clear_len;
+  uschar *ss = expand_string(s);
+  uschar *clear;
 
   /* Forced expansion failure is not an error; authentication is abandoned. On
   all but the first string, we have to abandon the authentication attempt by
@@ -204,19 +238,23 @@ while ((s = string_nextinlist(&text, &sep, big_buffer, big_buffer_size)) != NULL
 
   if (ss == NULL)
     {
-    char *ssave = string_copy(s);
+    uschar *ssave = string_copy(s);
     if (!first)
       {
-      if (smtp_write_command(outblock, FALSE, "*\r\n"))
+      if (smtp_write_command(outblock, FALSE, "*\r\n") >= 0)
         (void) smtp_read_response(inblock, US buffer, buffsize, '2', timeout);
       }
-    if (expand_string_forcedfail) return FORCEFAIL;
+    if (expand_string_forcedfail)
+      {
+      *buffer = 0;       /* No message */
+      return CANCELLED;
+      }
     string_format(buffer, buffsize, "expansion of \"%s\" failed in %s "
       "authenticator: %s", ssave, ablock->name, expand_string_message);
     return ERROR;
     }
 
-  len = (int)strlen(ss);
+  len = Ustrlen(ss);
 
   /* The character ^ is used as an escape for a binary zero character, which is
   needed for the PLAIN mechanism. It must be doubled if really needed. */
@@ -240,13 +278,15 @@ while ((s = string_nextinlist(&text, &sep, big_buffer, big_buffer_size)) != NULL
   if (first)
     {
     first = FALSE;
-    if (!smtp_write_command(outblock, FALSE, "AUTH %s%s%s\r\n",
-         ablock->public_name, (len == 0)? "" : " ", auth_b64encode(ss, len)))
+    if (smtp_write_command(outblock, FALSE, "AUTH %s%s%s\r\n",
+         ablock->public_name, (len == 0)? "" : " ",
+         auth_b64encode(ss, len)) < 0)
       return FAIL_SEND;
     }
   else
     {
-    if (!smtp_write_command(outblock, FALSE, "%s\r\n", auth_b64encode(ss, len)))
+    if (smtp_write_command(outblock, FALSE, "%s\r\n",
+          auth_b64encode(ss, len)) < 0)
       return FAIL_SEND;
     }
 
@@ -267,12 +307,40 @@ while ((s = string_nextinlist(&text, &sep, big_buffer, big_buffer_size)) != NULL
 
   if (text == NULL)
     {
-    if (smtp_write_command(outblock, FALSE, "*\r\n"))
+    if (smtp_write_command(outblock, FALSE, "*\r\n") >= 0)
       (void)smtp_read_response(inblock, US buffer, buffsize, '2', timeout);
     string_format(buffer, buffsize, "Too few items in client_send in %s "
       "authenticator", ablock->name);
     return ERROR;
     }
+
+  /* Now that we know we'll continue, we put the received data into $auth<n>,
+  if possible. First, decode it: buffer+4 skips over the SMTP status code. */
+
+  clear_len = auth_b64decode(buffer+4, &clear);
+
+  /* If decoding failed, the default is to terminate the authentication, and
+  return FAIL, with the SMTP response still in the buffer. However, if client_
+  ignore_invalid_base64 is set, we ignore the error, and put an empty string
+  into $auth<n>. */
+
+  if (clear_len < 0)
+    {
+    uschar *save_bad = string_copy(buffer);
+    if (!ob->client_ignore_invalid_base64)
+      {
+      if (smtp_write_command(outblock, FALSE, "*\r\n") >= 0)
+        (void)smtp_read_response(inblock, US buffer, buffsize, '2', timeout);
+      string_format(buffer, buffsize, "Invalid base64 string in server "
+        "response \"%s\"", save_bad);
+      return CANCELLED;
+      }
+    clear = US"";
+    clear_len = 0;
+    }
+
+  if (auth_var_idx < AUTH_VARS)
+    auth_vars[auth_var_idx++] = string_copy(clear);
   }
 
 /* Control should never actually get here. */
