@@ -1,10 +1,10 @@
-/* $Cambridge: exim/exim-src/src/directory.c,v 1.2 2005/01/04 10:00:42 ph10 Exp $ */
+/* $Cambridge: exim/exim-src/src/directory.c,v 1.4 2006/02/07 11:19:00 ph10 Exp $ */
 
 /*************************************************
 *     Exim - an Internet mail transport agent    *
 *************************************************/
 
-/* Copyright (c) University of Cambridge 1995 - 2005 */
+/* Copyright (c) University of Cambridge 1995 - 2006 */
 /* See the file NOTICE for conditions of use and distribution. */
 
 #include "exim.h"
@@ -74,13 +74,13 @@ while (c != 0 && *p != 0)
 
     /* Set the ownership if necessary. */
 
-    if (use_chown) Uchown(buffer, exim_uid, exim_gid);
+    if (use_chown) (void)Uchown(buffer, exim_uid, exim_gid);
 
     /* It appears that any mode bits greater than 0777 are ignored by
     mkdir(), at least on some operating systems. Therefore, if the mode
     contains any such bits, do an explicit mode setting. */
 
-    if ((mode & 0777000) != 0) Uchmod(buffer, mode);
+    if ((mode & 0777000) != 0) (void)Uchmod(buffer, mode);
     }
   *p++ = c;
   }
