@@ -1,15 +1,17 @@
+/* $Cambridge: exim/exim-src/src/lookups/oracle.h,v 1.3 2006/02/07 11:19:01 ph10 Exp $ */
+
 /*************************************************
 *     Exim - an Internet mail transport agent    *
 *************************************************/
 
-/* Copyright (c) University of Cambridge 1995 - 2004 */
+/* Copyright (c) University of Cambridge 1995 - 2006 */
 /* See the file NOTICE for conditions of use and distribution. */
 
 /* Header for the Oracle lookup functions */
 
 extern void   *oracle_open(uschar *, uschar **);
 extern int     oracle_find(void *, uschar *, uschar *, int, uschar **,
-                 uschar **);
+                 uschar **, BOOL *);
 extern void    oracle_tidy(void);
 extern uschar *oracle_quote(uschar *, uschar *);
 
