@@ -1,10 +1,10 @@
-/* $Cambridge: exim/exim-src/src/rda.c,v 1.10 2005/08/08 13:21:46 ph10 Exp $ */
+/* $Cambridge: exim/exim-src/src/rda.c,v 1.12 2006/02/07 11:19:00 ph10 Exp $ */
 
 /*************************************************
 *     Exim - an Internet mail transport agent    *
 *************************************************/
 
-/* Copyright (c) University of Cambridge 1995 - 2005 */
+/* Copyright (c) University of Cambridge 1995 - 2006 */
 /* See the file NOTICE for conditions of use and distribution. */
 
 /* This module contains code for extracting addresses from a forwarding list
@@ -770,14 +770,6 @@ if (read(fd, filtertype, sizeof(int)) != sizeof(int) ||
     read(fd, &yield, sizeof(int)) != sizeof(int) ||
     !rda_read_string(fd, error)) goto DISASTER;
 
-/* In the test harness, give the subprocess time to finish off and write
-any debugging output. */
-
-if (running_in_test_harness) millisleep(250);
-
-DEBUG(D_route)
-  debug_printf("rda_interpret: subprocess yield=%d error=%s\n", yield, *error);
-
 /* Read the contents of any syntax error blocks if we have a pointer */
 
 if (eblockp != NULL)
@@ -934,6 +926,9 @@ while ((rc = wait(&status)) != pid)
     }
   }
 
+DEBUG(D_route)
+  debug_printf("rda_interpret: subprocess yield=%d error=%s\n", yield, *error);
+
 if (had_disaster)
   {
   *error = string_sprintf("internal problem in %s: failure to transfer "
