@@ -1,10 +1,10 @@
-/* $Cambridge: exim/exim-src/src/routers/manualroute.h,v 1.2 2005/01/04 10:00:44 ph10 Exp $ */
+/* $Cambridge: exim/exim-src/src/routers/manualroute.h,v 1.4 2006/02/07 11:19:02 ph10 Exp $ */
 
 /*************************************************
 *     Exim - an Internet mail transport agent    *
 *************************************************/
 
-/* Copyright (c) University of Cambridge 1995 - 2005 */
+/* Copyright (c) University of Cambridge 1995 - 2006 */
 /* See the file NOTICE for conditions of use and distribution. */
 
 /* Header for the manualroute router */
@@ -31,7 +31,7 @@ extern manualroute_router_options_block manualroute_router_option_defaults;
 /* The main and initialization entry points for the router */
 
 extern int manualroute_router_entry(router_instance *, address_item *,
-  struct passwd *, BOOL, address_item **, address_item **,
+  struct passwd *, int, address_item **, address_item **,
   address_item **, address_item **);
 
 extern void manualroute_router_init(router_instance *);
