@@ -1,4 +1,4 @@
-/* $Cambridge: exim/exim-src/src/spam.c,v 1.4 2005/02/17 11:58:26 ph10 Exp $ */
+/* $Cambridge: exim/exim-src/src/spam.c,v 1.12 2006/07/03 15:19:44 ph10 Exp $ */
 
 /*************************************************
 *     Exim - an Internet mail transport agent    *
@@ -30,14 +30,25 @@ int spam(uschar **listptr) {
   FILE *mbox_file;
   int spamd_sock;
   uschar spamd_buffer[32600];
-  int i, j, offset;
+  int i, j, offset, result;
   uschar spamd_version[8];
   uschar spamd_score_char;
   double spamd_threshold, spamd_score;
   int spamd_report_offset;
   uschar *p,*q;
   int override = 0;
+  time_t start;
+  size_t read, wrote;
   struct sockaddr_un server;
+#ifndef NO_POLL_H
+  struct pollfd pollfd;
+#else                               /* Patch posted by Erik ? for OS X */
+  struct timeval select_tv;         /* and applied by PH */
+  fd_set select_fd;
+#endif
+
+  /* stop compiler warning */
+  result = result;
 
   /* find the username from the option list */
   if ((user_name = string_nextinlist(&list, &sep,
@@ -77,6 +88,7 @@ int spam(uschar **listptr) {
     return DEFER;
   };
 
+  start = time(NULL);
   /* socket does not start with '/' -> network socket */
   if (*spamd_address != '/') {
     time_t now = time(NULL);
@@ -114,7 +126,7 @@ int spam(uschar **listptr) {
     if (!num_servers) {
       log_write(0, LOG_MAIN|LOG_PANIC,
          "spam acl condition: no useable spamd server addresses in spamd_address configuration option.");
-      fclose(mbox_file);
+      (void)fclose(mbox_file);
       return DEFER;
     };
 
@@ -130,7 +142,7 @@ int spam(uschar **listptr) {
       if ( (spamd_sock = ip_socket(SOCK_STREAM, AF_INET)) < 0) {
         log_write(0, LOG_MAIN|LOG_PANIC,
            "spam acl condition: error creating IP socket for spamd");
-        fclose(mbox_file);
+        (void)fclose(mbox_file);
         return DEFER;
       };
 
@@ -153,8 +165,8 @@ int spam(uschar **listptr) {
         current_server = 0;
       if (current_server == start_server) {
         log_write(0, LOG_MAIN|LOG_PANIC, "spam acl condition: all spamd servers failed");
-        fclose(mbox_file);
-        close(spamd_sock);
+        (void)fclose(mbox_file);
+        (void)close(spamd_sock);
         return DEFER;
       };
     };
@@ -167,7 +179,7 @@ int spam(uschar **listptr) {
       log_write(0, LOG_MAIN|LOG_PANIC,
                 "malware acl condition: spamd: unable to acquire socket (%s)",
                 strerror(errno));
-      fclose(mbox_file);
+      (void)fclose(mbox_file);
       return DEFER;
     }
 
@@ -178,15 +190,15 @@ int spam(uschar **listptr) {
       log_write(0, LOG_MAIN|LOG_PANIC,
                 "malware acl condition: spamd: unable to connect to UNIX socket %s (%s)",
                 spamd_address, strerror(errno) );
-      fclose(mbox_file);
-      close(spamd_sock);
+      (void)fclose(mbox_file);
+      (void)close(spamd_sock);
       return DEFER;
     }
 
   }
 
   /* now we are connected to spamd on spamd_sock */
-  snprintf(CS spamd_buffer,
+  (void)string_format(spamd_buffer,
            sizeof(spamd_buffer),
            "REPORT SPAMC/1.2\r\nUser: %s\r\nContent-length: %ld\r\n\r\n",
            user_name,
@@ -194,42 +206,101 @@ int spam(uschar **listptr) {
 
   /* send our request */
   if (send(spamd_sock, spamd_buffer, Ustrlen(spamd_buffer), 0) < 0) {
-    close(spamd_sock);
+    (void)close(spamd_sock);
     log_write(0, LOG_MAIN|LOG_PANIC,
          "spam acl condition: spamd send failed: %s", strerror(errno));
-    fclose(mbox_file);
-    close(spamd_sock);
+    (void)fclose(mbox_file);
+    (void)close(spamd_sock);
     return DEFER;
   };
 
   /* now send the file */
+  /* spamd sometimes accepts conections but doesn't read data off
+   * the connection.  We make the file descriptor non-blocking so
+   * that the write will only write sufficient data without blocking
+   * and we poll the desciptor to make sure that we can write without
+   * blocking.  Short writes are gracefully handled and if the whole
+   * trasaction takes too long it is aborted.
+   * Note: poll() is not supported in OSX 10.2 and is reported to be
+   *       broken in more recent versions (up to 10.4).
+   */
+#ifndef NO_POLL_H
+  pollfd.fd = spamd_sock;
+  pollfd.events = POLLOUT;
+#endif
+  (void)fcntl(spamd_sock, F_SETFL, O_NONBLOCK);
   do {
-    j = fread(spamd_buffer,1,sizeof(spamd_buffer),mbox_file);
-    if (j > 0) {
-      i = send(spamd_sock,spamd_buffer,j,0);
-      if (i != j) {
-        log_write(0, LOG_MAIN|LOG_PANIC,
-          "spam acl condition: error/short send to spamd");
-        close(spamd_sock);
-        fclose(mbox_file);
-        return DEFER;
-      };
-    };
-  }
-  while (j > 0);
+    read = fread(spamd_buffer,1,sizeof(spamd_buffer),mbox_file);
+    if (read > 0) {
+      offset = 0;
+again:
+#ifndef NO_POLL_H
+      result = poll(&pollfd, 1, 1000);
 
-  fclose(mbox_file);
+/* Patch posted by Erik ? for OS X and applied by PH */
+#else
+      select_tv.tv_sec = 1;
+      select_tv.tv_usec = 0;
+      FD_ZERO(&select_fd);
+      FD_SET(spamd_sock, &select_fd);
+      result = select(spamd_sock+1, NULL, &select_fd, NULL, &select_tv);
+#endif
+/* End Erik's patch */
+
+      if (result == -1 && errno == EINTR)
+        goto again;
+      else if (result < 1) {
+        if (result == -1)
+          log_write(0, LOG_MAIN|LOG_PANIC,
+            "spam acl condition: %s on spamd socket", strerror(errno));
+        else {
+          if (time(NULL) - start < SPAMD_TIMEOUT)
+          goto again;
+          log_write(0, LOG_MAIN|LOG_PANIC,
+            "spam acl condition: timed out writing spamd socket");
+        }
+        (void)close(spamd_sock);
+        (void)fclose(mbox_file);
+        return DEFER;
+      }
+
+      wrote = send(spamd_sock,spamd_buffer + offset,read - offset,0);
+      if (wrote == -1)
+      {
+          log_write(0, LOG_MAIN|LOG_PANIC,
+            "spam acl condition: %s on spamd socket", strerror(errno));
+        (void)close(spamd_sock);
+        (void)fclose(mbox_file);
+        return DEFER;
+      }
+      if (offset + wrote != read) {
+        offset += wrote;
+        goto again;
+      }
+    }
+  }
+  while (!feof(mbox_file) && !ferror(mbox_file));
+  if (ferror(mbox_file)) {
+    log_write(0, LOG_MAIN|LOG_PANIC,
+      "spam acl condition: error reading spool file: %s", strerror(errno));
+    (void)close(spamd_sock);
+    (void)fclose(mbox_file);
+    return DEFER;
+  }
+
+  (void)fclose(mbox_file);
 
   /* we're done sending, close socket for writing */
   shutdown(spamd_sock,SHUT_WR);
 
-  /* read spamd response */
+  /* read spamd response using what's left of the timeout.
+   */
   memset(spamd_buffer, 0, sizeof(spamd_buffer));
   offset = 0;
   while((i = ip_recv(spamd_sock,
                      spamd_buffer + offset,
                      sizeof(spamd_buffer) - offset - 1,
-                     SPAMD_READ_TIMEOUT)) > 0 ) {
+                     SPAMD_TIMEOUT - time(NULL) + start)) > 0 ) {
     offset += i;
   }
 
@@ -237,12 +308,12 @@ int spam(uschar **listptr) {
   if((i <= 0) && (errno != 0)) {
     log_write(0, LOG_MAIN|LOG_PANIC,
          "spam acl condition: error reading from spamd socket: %s", strerror(errno));
-    close(spamd_sock);
+    (void)close(spamd_sock);
     return DEFER;
   }
 
   /* reading done */
-  close(spamd_sock);
+  (void)close(spamd_sock);
 
   /* dig in the spamd output and put the report in a multiline header, if requested */
   if( sscanf(CS spamd_buffer,"SPAMD/%s 0 EX_OK\r\nContent-length: %*u\r\n\r\n%lf/%lf\r\n%n",
@@ -306,12 +377,12 @@ int spam(uschar **listptr) {
   spam_bar = spam_bar_buffer;
 
   /* create "float" spam score */
-  snprintf(CS spam_score_buffer, sizeof(spam_score_buffer),"%.1f", spamd_score);
+  (void)string_format(spam_score_buffer, sizeof(spam_score_buffer),"%.1f", spamd_score);
   spam_score = spam_score_buffer;
 
   /* create "int" spam score */
   j = (int)((spamd_score + 0.001)*10);
-  snprintf(CS spam_score_int_buffer, sizeof(spam_score_int_buffer), "%d", j);
+  (void)string_format(spam_score_int_buffer, sizeof(spam_score_int_buffer), "%d", j);
   spam_score_int = spam_score_int_buffer;
 
   /* compare threshold against score */
