@@ -1,4 +1,4 @@
-/* $Cambridge: exim/exim-src/src/spf.c,v 1.3 2005/02/17 11:58:26 ph10 Exp $ */
+/* $Cambridge: exim/exim-src/src/spf.c,v 1.6 2005/06/27 15:28:45 tom Exp $ */
 
 /*************************************************
 *     Exim - an Internet mail transport agent    *
@@ -13,54 +13,50 @@
 #include "exim.h"
 #ifdef EXPERIMENTAL_SPF
 
-/* #include "spf.h" */
-
-SPF_config_t        spfcid = NULL;
-SPF_dns_config_t    spfdcid_resolv = NULL;
-SPF_dns_config_t    spfdcid = NULL;
+/* must be kept in numeric order */
+static spf_result_id spf_result_id_list[] = {
+  { US"invalid", 0},
+  { US"neutral", 1 },
+  { US"pass", 2 },
+  { US"fail", 3 },
+  { US"softfail", 4 },
+  { US"none", 5 },
+  { US"err_temp", 6 },
+  { US"err_perm", 7 }
+};
 
+SPF_server_t    *spf_server = NULL;
+SPF_request_t   *spf_request = NULL;
+SPF_response_t  *spf_response = NULL;
+SPF_response_t  *spf_response_2mx = NULL;
 
 /* spf_init sets up a context that can be re-used for several
    messages on the same SMTP connection (that come from the
    same host with the same HELO string) */
 
 int spf_init(uschar *spf_helo_domain, uschar *spf_remote_addr) {
-  uschar *p;
 
-  /* paranoia */
-  spfcid = NULL;
-  spfdcid_resolv = NULL;
-  spfdcid = NULL;
+  spf_server = SPF_server_new(SPF_DNS_CACHE, 0);
 
-  spfcid = SPF_create_config();
-  if ( spfcid == NULL ) {
-    debug_printf("spf: SPF_create_config() failed.\n");
-	  return 0;
+  if ( spf_server == NULL ) {
+    debug_printf("spf: SPF_server_new() failed.\n");
+    return 0;
   }
 
-  /* set up resolver */
-  spfdcid_resolv = SPF_dns_create_config_resolv(NULL, 0);
-  spfdcid = SPF_dns_create_config_cache(spfdcid_resolv, 8, 0);
+  spf_request = SPF_request_new(spf_server);
 
-  if (spfdcid == NULL) {
-    debug_printf("spf: SPF_dns_create_config_cache() failed.\n");
-    spfcid = NULL;
-    spfdcid_resolv = NULL;
-	  return 0;
+  if (SPF_request_set_ipv4_str(spf_request, spf_remote_addr)) {
+    debug_printf("spf: SPF_request_set_ipv4_str() failed.\n");
+    spf_server = NULL;
+    spf_request = NULL;
+    return 0;
   }
 
-  if (SPF_set_ip_str(spfcid, spf_remote_addr)) {
-    debug_printf("spf: SPF_set_ip_str() failed.\n");
-    spfcid = NULL;
-    spfdcid_resolv = NULL;
-	  return 0;
-  }
-
-  if (SPF_set_helo_dom(spfcid, spf_helo_domain)) {
+  if (SPF_request_set_helo_dom(spf_request, spf_helo_domain)) {
     debug_printf("spf: SPF_set_helo_dom() failed.\n");
-    spfcid = NULL;
-    spfdcid_resolv = NULL;
-	  return 0;
+    spf_server = NULL;
+    spf_request = NULL;
+    return 0;
   }
 
   return 1;
@@ -76,31 +72,30 @@ int spf_process(uschar **listptr, uschar *spf_envelope_sender) {
   uschar *list = *listptr;
   uschar *spf_result_id;
   uschar spf_result_id_buffer[128];
-  SPF_output_t spf_output;
-  int rc = SPF_RESULT_ERROR;
+  int rc = SPF_RESULT_PERMERROR;
 
-  if (!(spfcid && spfdcid)) {
+  if (!(spf_server && spf_request)) {
     /* no global context, assume temp error and skip to evaluation */
-    rc = SPF_RESULT_ERROR;
+    rc = SPF_RESULT_PERMERROR;
     goto SPF_EVALUATE;
   };
 
-  if (SPF_set_env_from(spfcid, spf_envelope_sender)) {
+  if (SPF_request_set_env_from(spf_request, spf_envelope_sender)) {
     /* Invalid sender address. This should be a real rare occurence */
-    rc = SPF_RESULT_ERROR;
+    rc = SPF_RESULT_PERMERROR;
     goto SPF_EVALUATE;
   }
 
   /* get SPF result */
-  spf_output = SPF_result(spfcid, spfdcid);
+  SPF_request_query_mailfrom(spf_request, &spf_response);
 
   /* set up expansion items */
-  spf_header_comment     = spf_output.header_comment ? (uschar *)spf_output.header_comment : NULL;
-  spf_received           = spf_output.received_spf ? (uschar *)spf_output.received_spf : NULL;
-  spf_result             = (uschar *)SPF_strresult(spf_output.result);
-  spf_smtp_comment       = spf_output.smtp_comment ? (uschar *)spf_output.smtp_comment : NULL;
+  spf_header_comment     = (uschar *)SPF_response_get_header_comment(spf_response);
+  spf_received           = (uschar *)SPF_response_get_received_spf(spf_response);
+  spf_result             = (uschar *)SPF_strresult(SPF_response_result(spf_response));
+  spf_smtp_comment       = (uschar *)SPF_response_get_smtp_comment(spf_response);
 
-  rc = spf_output.result;
+  rc = SPF_response_result(spf_response);
 
   /* We got a result. Now see if we should return OK or FAIL for it */
   SPF_EVALUATE:
@@ -128,4 +123,3 @@ int spf_process(uschar **listptr, uschar *spf_envelope_sender) {
 }
 
 #endif
-
