@@ -1,10 +1,10 @@
-/* $Cambridge: exim/exim-src/src/transports/smtp.h,v 1.4 2005/02/17 11:58:27 ph10 Exp $ */
+/* $Cambridge: exim/exim-src/src/transports/smtp.h,v 1.8 2006/02/28 12:42:47 ph10 Exp $ */
 
 /*************************************************
 *     Exim - an Internet mail transport agent    *
 *************************************************/
 
-/* Copyright (c) University of Cambridge 1995 - 2005 */
+/* Copyright (c) University of Cambridge 1995 - 2006 */
 /* See the file NOTICE for conditions of use and distribution. */
 
 /* Private structure for the private options and other private data. */
@@ -34,6 +34,7 @@ typedef struct {
   int     hosts_max_try;
   int     hosts_max_try_hardlimit;
   BOOL    allow_localhost;
+  BOOL    authenticated_sender_force;
   BOOL    gethostbyname;
   BOOL    dns_qualify_single;
   BOOL    dns_search_parents;
@@ -41,6 +42,7 @@ typedef struct {
   BOOL    hosts_override;
   BOOL    hosts_randomize;
   BOOL    keepalive;
+  BOOL    lmtp_ignore_quota;
   BOOL    retry_include_ip_address;
   #ifdef SUPPORT_TLS
   uschar *tls_certificate;
@@ -50,6 +52,14 @@ typedef struct {
   uschar *tls_verify_certificates;
   BOOL    tls_tempfail_tryclear;
   #endif
+  #ifdef EXPERIMENTAL_DOMAINKEYS
+  uschar *dk_domain;
+  uschar *dk_private_key;
+  uschar *dk_selector;
+  uschar *dk_canon;
+  uschar *dk_headers;
+  uschar *dk_strict;
+  #endif
 } smtp_transport_options_block;
 
 /* Data for reading the private options. */
